package com.qucheng.game.data.oldsystem.serialization;

import com.alibaba.fastjson2.JSONObject;
import com.ververica.cdc.debezium.DebeziumDeserializationSchema;
import io.debezium.data.Envelope;
import org.apache.flink.api.common.typeinfo.BasicTypeInfo;
import org.apache.flink.api.common.typeinfo.TypeInformation;
import org.apache.flink.util.Collector;
import org.apache.kafka.connect.data.Field;
import org.apache.kafka.connect.data.Struct;
import org.apache.kafka.connect.source.SourceRecord;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.Date;

public class CustomerDeserializationSchema implements DebeziumDeserializationSchema<String> {

    //格式化后目标数据长下边这样

    /**
     * {
     * "db":"",
     * "tableName":"",
     * "before":{"id":"1001","name":""...},
     * "after":{"id":"1001","name":""...},
     * "op":""
     * }
     */


    @Override
    public void deserialize(SourceRecord sourceRecord, Collector<String> collector) throws Exception {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        SimpleDateFormat sdfs = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //创建JSON对象用于封装结果数据
        JSONObject result = new JSONObject();

        //获取库名&表名
        String topic = sourceRecord.topic();
        String[] fields = topic.split("\\.");

        result.put("db", fields[1]);
        result.put("tableName", fields[2]);

        //获取主键
        Struct key = (Struct) sourceRecord.key();
        JSONObject keyJson = new JSONObject();
        if (key != null) {
            for (Field field : key.schema().fields()) {
                String fieldName = field.name();
                String schemaName = field.schema().name();

                Object fieldValue = key.get(field);
                String type = field.schema().type().name().toLowerCase();
                //转换时间戳
                if ("int32".equals(type) && "io.debezium.time.Date".equals(schemaName)) {
//                    System.out.println("(int)fieldValue = "+(int)fieldValue);
                    int day = (int) fieldValue;
                    Long second = day * 24 * 60 * 60L;
                    Date date = new Date();
                    date.setTime(second * 1000);
                    String dateStr = sdf.format(date);
                    keyJson.put(fieldName, sdf.format(date).toString());
                    //转换时间戳
                } else if ("int64".equals(type) && "io.debezium.time.Timestamp".equals(schemaName)) {
                    long times = (long) fieldValue;
                    String dateTime = sdfs.format(times - 8 * 60 * 60 * 1000);

                    keyJson.put(fieldName, dateTime);
                    //转换BLOB类型数据
                } else if ("bytes".equals(type) && !"org.apache.kafka.connect.data.Decimal".equals(schemaName)) {
                    ByteBuffer byteBuffer2 = (ByteBuffer) fieldValue;
                    String s = StandardCharsets.UTF_8.decode(byteBuffer2).toString();
                    keyJson.put(fieldName, s);
                } else {
                    keyJson.put(fieldName, fieldValue);
                }
            }
        }

        //加入key，用作更新和删除
        result.put("key", keyJson);

        Struct value = (Struct) sourceRecord.value();

        //获取after数据
        Struct after = value.getStruct("after");
        JSONObject afterJson = new JSONObject();
        if (after != null) {
            for (Field field : after.schema().fields()) {
                String fieldName = field.name();
                String schemaName = field.schema().name();

                Object fieldValue = after.get(field);
                String type = field.schema().type().name().toLowerCase();
                //转换时间戳
                if(fieldValue != null){
                    if ("int32".equals(type) && "io.debezium.time.Date".equals(schemaName)) {
                        int day = (int) fieldValue;
                        Long second = day * 24 * 60 * 60L;
                        Date date = new Date();
                        date.setTime(second * 1000);
                        afterJson.put(fieldName, sdf.format(date).toString());
                        //转换时间戳
                    } else if ( "int64".equals(type) && "io.debezium.time.Timestamp".equals(schemaName)) {
                        long times = (long) fieldValue;
                        String dateTime = sdfs.format(times - 8 * 60 * 60 * 1000);

                        afterJson.put(fieldName, dateTime);
                        //转换BLOB类型数据
                    } else if ("bytes".equals(type) && !"org.apache.kafka.connect.data.Decimal".equals(schemaName)) {
                        ByteBuffer byteBuffer2 = (ByteBuffer) fieldValue;
                        String s = StandardCharsets.UTF_8.decode(byteBuffer2).toString();
                        afterJson.put(fieldName, s);
                    } else {
                        afterJson.put(fieldName, fieldValue);
                    }
                }else{
                    afterJson.put(fieldName, fieldValue);
                }

            }
        }

        result.put("after", afterJson);

        //获取操作类型
        Envelope.Operation operation = Envelope.operationFor(sourceRecord);

        String type = operation.toString().toLowerCase();
        if ("create".equals(type) || "read".equals(type)) {
            type = "insert";
        } else if ("delete".equals(type)) {
            type = "delete";
        } else if ("update".equals(type)) {
            type = "update";
        }

        //加入操作类型
        result.put("type", type);

        //输出数据
        collector.collect(result.toJSONString());

    }

    @Override
    public TypeInformation<String> getProducedType() {
        return BasicTypeInfo.STRING_TYPE_INFO;
    }
}
