package com.qucheng.game.data.oldsystem.util;

import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.connector.base.DeliveryGuarantee;
import org.apache.flink.connector.kafka.sink.KafkaRecordSerializationSchema;
import org.apache.flink.connector.kafka.sink.KafkaSink;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;

public class MyKafkaUtil {



        //往kafka写入数据
    public static KafkaSink<String> getKafkaSink(String KAFKA_BROKERS,String topic) {
        return KafkaSink.<String>builder()
                .setBootstrapServers(KAFKA_BROKERS)
                .setRecordSerializer(KafkaRecordSerializationSchema.builder()
                        .setTopic(topic)
                        .setValueSerializationSchema(new SimpleStringSchema())
                        .build())
                .setDeliverGuarantee(DeliveryGuarantee.AT_LEAST_ONCE)
                .build();
    }

    //读取kafka数据
    public static KafkaSource<String> getKafkaSource(String KAFKA_BROKERS,String topic, String groupId, OffsetsInitializer offsets) {
        /**
         * 以下是kafka的offset设置
         *      OffsetsInitializer.committedOffsets()
         *      OffsetsInitializer.committedOffsets(OffsetResetStrategy.EARLIEST)
         *      OffsetsInitializer.timestamp(1592323200L)
         *      OffsetsInitializer.earliest()
         *      OffsetsInitializer.latest()
         */
        return KafkaSource.<String>builder()
                .setProperty("partition.discovery.interval.ms", "60000")
//                .setProperty("enable.auto.commit","false")
//                .setProperty("auto.commit.interval.ms","1000")
                .setBootstrapServers(KAFKA_BROKERS)
                .setTopics(topic)
                .setGroupId(groupId)
                .setValueOnlyDeserializer(new SimpleStringSchema())
                .setStartingOffsets(offsets)
                .build();
    }

    public static String getKafkaDDL(String KAFKA_BROKERS,String topic, String groupId,String consumption) {
        return " with ('connector' = 'kafka', " +
                " 'topic' = '" + topic + "'," +
                " 'properties.bootstrap.servers' = '" + KAFKA_BROKERS + "', " +
                " 'properties.group.id' = '" + groupId + "', " +
                " 'format' = 'json', " +

                " 'scan.startup.mode' = '" + consumption + "')";
    }

    /**
     * Kafka-Sink DDL 语句
     *
     * @param topic 输出到 Kafka 的目标主题
     * @return 拼接好的 Kafka-Sink DDL 语句
     */
    public static String getKafkaSinkDDL(String KAFKA_BROKERS,String topic) {
        return " WITH ( " +
                "  'connector' = 'kafka', " +
                "  'topic' = '" + topic + "', " +
                "  'properties.bootstrap.servers' = '" + KAFKA_BROKERS + "', " +
                "  'format' = 'json' " +
                ")";
    }

    /**
     * UpsertKafka-Sink DDL 语句
     *
     * @param topic 输出到 Kafka 的目标主题
     * @return 拼接好的 UpsertKafka-Sink DDL 语句
     */
    public static String getUpsertKafkaDDL(String KAFKA_BROKERS,String topic) {
        return " WITH ( " +
                "  'connector' = 'upsert-kafka', " +
                "  'topic' = '" + topic + "', " +
                "  'properties.bootstrap.servers' = '" + KAFKA_BROKERS + "', " +
                "  'key.format' = 'json', " +
                "  'value.format' = 'json' " +
                ")";
    }

    /**
     * topic_db主题的  Kafka-Source DDL 语句
     *
     * @param groupId 消费者组
     * @return 拼接好的 Kafka 数据源 DDL 语句
     */
    public static String getTopicDb(String KAFKA_BROKERS,String topic,String groupId,String consumption) {
        return "CREATE TABLE topic_db ( " +
                "  `db` STRING, " +
                "  `tableName` STRING, " +
                "  `type` STRING, " +
                "  `key` MAP<STRING,STRING>, " +
                "  `after` MAP<STRING,STRING>, " +
                "  `pt` AS PROCTIME()" +
                ") " + getKafkaDDL(KAFKA_BROKERS,topic, groupId,consumption);
    }

    public static String getTopicDb1(String KAFKA_BROKERS,String topic,String groupId,String consumption) {
        return "CREATE TABLE topic_db1 ( " +
                "  `db` STRING, " +
                "  `tableName` STRING, " +
                "  `type` STRING, " +
                "  `key` MAP<STRING,STRING>, " +
                "  `after` MAP<STRING,STRING>, " +
                "  `pt` AS PROCTIME()" +
                ") " + getKafkaDDL(KAFKA_BROKERS,topic, groupId,consumption);
    }
}
